package com.autoussd.autoussdflutter

import android.util.Log
import androidx.activity.ComponentActivity
import androidx.annotation.NonNull
import com.autoussd.AutoUssd
import com.autoussd.models.DeviceSimNetworksListener
import com.autoussd.models.Network
import com.autoussd.models.OverlayThemeData
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result as MethodChannelResult

class AutoUssdFlutterPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
  companion object {
    private const val TAG = "AutoUssdFlutterPlugin"
    private const val NATIVE_COUNT_LISTENER_NAME = "autoussdCount"
    private const val NATIVE_RESULT_LISTENER_NAME = "autoussdResult"
    private const val NATIVE_EXECUTE_SESSION_METHOD = "executeSession"
    private const val NATIVE_GET_DEVICE_SIM_NETWORKS_METHOD = "getDeviceSimNetworks"
    private const val NATIVE_SET_OVERLAY_THEME_METHOD = "setOverlayTheme"
    private const val FLUTTER_COUNT_CALLBACK_METHOD = "onCount"
    private const val FLUTTER_RESULT_CALLBACK_METHOD = "onResult"
    private const val FLUTTER_GET_DEVICE_SIM_NETWORKS_ERROR = "getDeviceSimNetworksError"
  }

  private lateinit var channel: MethodChannel

  override fun onAttachedToEngine(
    @NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding
  ) {
    Log.d(TAG, "On Attached To Engine")

    channel = MethodChannel(
      flutterPluginBinding.binaryMessenger,
      "autoussdflutter"
    )

    channel.setMethodCallHandler(this)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    Log.d(TAG, "On Attached To Activity")

    try {
      AutoUssd.init(binding.activity as ComponentActivity)

      AutoUssd.getInstance().registerSessionCountListener(NATIVE_COUNT_LISTENER_NAME) {
        binding.activity.runOnUiThread {
          channel.invokeMethod(
            FLUTTER_COUNT_CALLBACK_METHOD,
            it
          )
        }
      }

      AutoUssd.getInstance().registerSessionResultListener(NATIVE_RESULT_LISTENER_NAME) {
        binding.activity.runOnUiThread {
          channel.invokeMethod(
            FLUTTER_RESULT_CALLBACK_METHOD,
            listOf(
              it.sessionId,
              it.status.toString(),
              it.description,
              it.lastContent,
              it.parser,
            )
          )
        }
      }

      Log.d(TAG, "SDK initialized")
    } catch (err: Exception) {
      err.printStackTrace()
      Log.e(TAG, err.message ?: "Could not create instance of AutoUssd")
    }
  }

  override fun onDetachedFromActivity() {
    Log.d(TAG, "On Detached From Activity")

    AutoUssd.getInstance().unregisterSessionCountListener(NATIVE_COUNT_LISTENER_NAME)
    AutoUssd.getInstance().unregisterSessionResultListener(NATIVE_RESULT_LISTENER_NAME)
    AutoUssd.getInstance().dispose()
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    Log.d(TAG, "On Reattached To Activity For Config Changes")

    channel.setMethodCallHandler(this)
  }

  override fun onDetachedFromActivityForConfigChanges() {
    Log.d(TAG, "On Detached To Activity For Config Changes")

    channel.setMethodCallHandler(null)
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    Log.d(TAG, "On Detached From Engine")

    channel.setMethodCallHandler(null)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannelResult) {
    Log.d(TAG, "On Method Call")

    if (call.method == NATIVE_EXECUTE_SESSION_METHOD) {
      val sessionId = call.argument<String?>("sessionId")!!
      val variables = call.argument<Map<String, String>?>("variables")?.toMap()
      val network = call.argument<HashMap<String, String>?>("network")

      if (network == null) {
        AutoUssd.getInstance().executeSession(
          sessionId,
          variables,
          null,
        )
      } else {
        AutoUssd.getInstance().executeSession(
          sessionId,
          variables,
          Network(
            network["mcc"],
            network["mnc"],
          ),
        )
      }
    } else if (call.method == NATIVE_GET_DEVICE_SIM_NETWORKS_METHOD) {
      AutoUssd.getInstance().getDeviceSimNetworks(object : DeviceSimNetworksListener {
        override fun onResult(networks: List<Network>) {
          result.success(
            networks.map {
              mapOf(
                "name" to it.name,
                "mcc" to it.mcc,
                "mnc" to it.mnc,
                "countryName" to it.countryName,
                "countryIsoCode" to it.countryIsoCode,
                "countryDialCode" to it.countryDialCode,
              )
            }
          )
        }

        override fun onPermissionDenied() {
          result.error(
            FLUTTER_GET_DEVICE_SIM_NETWORKS_ERROR,
            null,
            null
          )
        }
      })
    } else if (call.method == NATIVE_SET_OVERLAY_THEME_METHOD) {
      val themeDataMap = call.argument<Map<String, Any>?>("theme")?.toMap()

      if (themeDataMap != null) {
        AutoUssd.getInstance().setOverlayTheme(
          OverlayThemeData(
            backgroundColor = (themeDataMap[OverlayThemeData.BACKGROUND_COLOR_KEY] as Long?)?.toInt(),
            textColor = (themeDataMap[OverlayThemeData.TEXT_COLOR_KEY] as Long?)?.toInt(),
            progressColor = (themeDataMap[OverlayThemeData.PROGRESS_COLOR_KEY] as Long?)?.toInt(),
            buttonBackgroundColor = (themeDataMap[OverlayThemeData.BUTTON_BACKGROUND_COLOR_KEY] as Long?)?.toInt(),
            buttonTextColor = (themeDataMap[OverlayThemeData.BUTTON_TEXT_COLOR_KEY] as Long?)?.toInt(),
            inputBackgroundColor = (themeDataMap[OverlayThemeData.INPUT_BACKGROUND_COLOR_KEY] as Long?)?.toInt(),
            inputOutlineColor = (themeDataMap[OverlayThemeData.INPUT_OUTLINE_COLOR_KEY] as Long?)?.toInt(),
            inputTextColor = (themeDataMap[OverlayThemeData.INPUT_TEXT_COLOR_KEY] as Long?)?.toInt(),
            inputHintColor = (themeDataMap[OverlayThemeData.INPUT_HINT_COLOR_KEY] as Long?)?.toInt(),
          )
        )
      }
    } else result.notImplemented()
  }
}
